% File: load.pl
% -------------
%
% Central file to load all modules needed to run the examples.
%
% This file provides a predicate run_example/3 to easily run the examples
% bundled in this package. To run the first example, type
%
%   ?- run_example(example1, MaxCon, MinInc).
%
% This will consult the file "example1.pl" that defines the example and
% then call compute_blends/6 to find maximal consistent and minimal
% inconsistent blends. The results are unified with MaxCon and MinInc
% respectively.
%
% Additional examples can be provided by implementing additional example
% files. For the structure of such a file, refer to "example1.pl" and
% "example2.pl". Briefly, such a file should define predicates
% background/1, axioms/1, and init/1 to provide a background theory,
% as set of axioms that may be included in the blend and the set
% of initial theories to start exploration. In addition, a predicate
% example/2 should be provided, that can translate axiom names into
% logical formulas understood by the underlying reasoner.





%! theory(?Blend:list,?Theory:list)
%
% Make a theory from a blend by adding formulase from the backround theory
% (provided by background/1).
%
% @param Blend a list for axiom names.
% @param Theory a list for axiom names consisting of background/1 axiom
%   names and the axioms names from Blend.
theory(Blend,Theory) :-
    background(Background),
    blend(Blend,BlendAxioms),
    append(Background,BlendAxioms,Theory).


% Translate labels to formula strings that can be used by the
% consistency checker. The translation is based on the predicate
% example/2.

lookup_example([], []) :- !.

lookup_example([Name|Names], [Formula|Formulae]) :-
    example(Name,Formula),
    lookup_example(Names,Formulae).



lookup_theory([], []) :- !.

lookup_theory([Name|Names], [Theory|Theories]) :-
    example(Name,Theory),
    lookup_example(Names,Theories).



:- use_module(prover9).


%! consistent(+AxiomNames:list) is det
%
% Check if a given set of axioms is consistent.
%
% @param AxiomNames A list of axiom names. Each name will be translated
%     into a formula in a format understood by prover9, using a mapping
%     provided by the predicate example/2.
%
consistent(AxiomNames) :-
    lookup_theory(AxiomNames, Theory), !,
    prover9_consistent(Theory).


:- use_module(blending).

:- dynamic example/2, background/1, axioms/1, init/1, blend/2.
:- multifile example/2, background/1, axioms/1, init/1, blend/2.


%! run_example(Example,-MaxCon,-MinInc) is det
%
% Consult the file Example to provide the example data and then run
% the compute_blends/5 algorithm.
%
% @param Example name of a prolog file containing an example.
%     The file should define background/1, blend/1 and init/1 to
%     provide the parameters for compute_blends/5. It should
%     also define example/2 to map axiom names to logical formulas.
%
run_example(Example,MaxCon,MinInc) :-
    retractall(example(_,_)),
    retractall(background(_)),
    retractall(axioms(_)),
    retractall(init(_)),
    retractall(blend(_,_)),
    consult(Example),
    background(Base),
    axioms(Axioms),
    init(Init),
    compute_blends(Base,Axioms,Init,MaxCon,MinInc,both).

